@@ -1,29 +1,25 @@
 <?php
 
-// {$Id: digestauth.php 371 2009-03-29 19:40:47Z evertpot $} //
-
 // !!!! Make sure the Sabre directory is in the include_path !!!
 // example:
-// set_include_dir('lib/' . PATH_SEPARATOR . get_include_path(); 
+// set_include_path('lib/' . PATH_SEPARATOR . get_include_path());
 
 // settings
 date_default_timezone_set('Canada/Eastern');
 
 // Files we need
-require_once 'Sabre.includes.php';
+require_once 'vendor/autoload.php';
 
 $u = 'admin';
 $p = '1234';
 
-$auth = new Sabre_HTTP_DigestAuth();
+$auth = new \Sabre\HTTP\DigestAuth();
 $auth->init();
 
-if ($auth->getUsername() != $u || !$auth->validatePassword($p)) { 
+if ($auth->getUsername() != $u || !$auth->validatePassword($p)) {
 
     $auth->requireLogin();
     echo "Authentication required\n";
     die();
 
 }
-
-?>
