@@ -1,20 +1,19 @@
 <?php
 
+namespace Sabre\DAV;
+
 /**
  * IQuota interface
  *
  * Implement this interface to add the ability to return quota information. The ObjectTree
- * will check for quota information on any given node. If the information is not available it will 
+ * will check for quota information on any given node. If the information is not available it will
  * attempt to fetch the information from the root node.
- * 
- * @package Sabre
- * @subpackage DAV
- * @version $Id: IQuota.php 385 2009-03-30 01:15:51Z evertpot $
- * @copyright Copyright (C) 2007-2009 Rooftop Solutions. All rights reserved.
- * @author Evert Pot (http://www.rooftopsolutions.nl/) 
+ *
+ * @copyright Copyright (C) 2007-2013 fruux GmbH (https://fruux.com/).
+ * @author Evert Pot (http://evertpot.com/)
  * @license http://code.google.com/p/sabredav/wiki/License Modified BSD License
  */
-interface Sabre_DAV_IQuota extends Sabre_DAV_IDirectory {
+interface IQuota extends ICollection {
 
     /**
      * Returns the quota information
@@ -22,7 +21,7 @@ interface Sabre_DAV_IQuota extends Sabre_DAV_IDirectory {
      * This method MUST return an array with 2 values, the first being the total used space,
      * the second the available space (in bytes)
      */
-    function getQuotaInfo(); 
+    function getQuotaInfo();
 
 }
 
